@@ -12,10 +12,6 @@
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  * GNU General Public License for more details.
- *
- * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
  */
 
 
@@ -94,6 +90,7 @@ static int ms_talloc_destructor(struct GprsMs *ms);
 struct GprsMs *ms_alloc(struct gprs_rlcmac_bts *bts, uint32_t tlli)
 {
 	struct GprsMs *ms = talloc_zero(tall_pcu_ctx, struct GprsMs);
+	OSMO_ASSERT(bts);
 
 	talloc_set_destructor(ms, ms_talloc_destructor);
 
@@ -114,22 +111,16 @@ struct GprsMs *ms_alloc(struct gprs_rlcmac_bts *bts, uint32_t tlli)
 	LOGP(DRLCMAC, LOGL_INFO, "Creating MS object, TLLI = 0x%08x\n", tlli);
 
 	ms->imsi[0] = '\0';
-	memset(&ms->timer, 0, sizeof(ms->timer));
-	ms->timer.cb = ms_release_timer_cb;
-	llc_queue_init(&ms->llc_queue);
+	osmo_timer_setup(&ms->timer, ms_release_timer_cb, NULL);
+	llc_queue_init(&ms->llc_queue, ms);
 
 	ms_set_mode(ms, GPRS);
 
-	if (ms->bts)
-		codel_interval = the_pcu->vty.llc_codel_interval_msec;
+	codel_interval = the_pcu->vty.llc_codel_interval_msec;
+	if (codel_interval == LLC_CODEL_USE_DEFAULT)
+		codel_interval = GPRS_CODEL_SLOW_INTERVAL_MS;
+	llc_queue_set_codel_interval(&ms->llc_queue, codel_interval);
 
-	if (codel_interval) {
-		if (codel_interval == LLC_CODEL_USE_DEFAULT)
-			codel_interval = GPRS_CODEL_SLOW_INTERVAL_MS;
-		ms->codel_state = talloc(ms, struct gprs_codel);
-		gprs_codel_init(ms->codel_state);
-		gprs_codel_set_interval(ms->codel_state, codel_interval);
-	}
 	ms->last_cs_not_low = now_msec();
 	ms->app_info_pending = false;
 
