@@ -12,10 +12,6 @@
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  * GNU General Public License for more details.
- *
- * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
  */
 
 #pragma once
@@ -96,8 +92,6 @@ struct GprsMs {
 	uint8_t reserved_dl_slots;
 	uint8_t reserved_ul_slots;
 	struct gprs_rlcmac_trx *current_trx;
-
-	struct gprs_codel *codel_state;
 	enum mcs_kind mode;
 
 	struct rate_ctr_group *ctrs;
@@ -222,11 +216,6 @@ static inline void ms_set_timeout(struct GprsMs *ms, unsigned secs)
 	ms->delay = secs;
 }
 
-static inline struct gprs_codel *ms_codel_state(const struct GprsMs *ms)
-{
-	return ms->codel_state;
-}
-
 static inline unsigned ms_nack_rate_dl(const struct GprsMs *ms)
 {
 	return ms->nack_rate_dl;
