@@ -11,17 +11,16 @@
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  * GNU General Public License for more details.
- *
- * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
  */
 
 #include "gprs_rlcmac.h"
 #include "gprs_debug.h"
 #include "tbf.h"
 #include "tbf_ul.h"
+#include "tbf_dl.h"
 #include "bts.h"
+#include "gprs_ms.h"
+#include "bts_pch_timer.h"
 
 #include <string.h>
 #include <stdio.h>
@@ -32,6 +31,7 @@ extern "C" {
 #include <osmocom/core/msgb.h>
 #include <osmocom/core/talloc.h>
 #include <osmocom/core/utils.h>
+#include <osmocom/core/fsm.h>
 }
 
 /* globals used by the code */
@@ -41,28 +41,27 @@ bool spoof_mnc_3_digits = false;
 
 static gprs_rlcmac_tbf *tbf_alloc(struct gprs_rlcmac_bts *bts,
 		GprsMs *ms, gprs_rlcmac_tbf_direction dir,
-		uint8_t use_trx,
-		uint8_t ms_class, uint8_t egprs_ms_class, bool single_slot)
+		uint8_t use_trx, bool single_slot)
 {
+	OSMO_ASSERT(ms != NULL);
+
 	if (dir == GPRS_RLCMAC_UL_TBF)
-		return tbf_alloc_ul_tbf(bts, ms, use_trx,
-			ms_class, egprs_ms_class, single_slot);
+		return tbf_alloc_ul_tbf(bts, ms, use_trx, single_slot);
 	else
-		return tbf_alloc_dl_tbf(bts, ms, use_trx,
-			ms_class, egprs_ms_class, single_slot);
+		return tbf_alloc_dl_tbf(bts, ms, use_trx, single_slot);
 }
 
-static void check_tfi_usage(BTS *the_bts)
+static void check_tfi_usage(struct gprs_rlcmac_bts *bts)
 {
 	int pdch_no;
 
 	struct gprs_rlcmac_tbf *tfi_usage[8][8][2][32] = {{{{NULL}}}};
-	LListHead<gprs_rlcmac_tbf> *tbf_lists[2] = {
-		&the_bts->ul_tbfs(),
-		&the_bts->dl_tbfs()
+	struct llist_head *tbf_lists[2] = {
+		&bts->trx[0].ul_tbfs,
+		&bts->trx[0].dl_tbfs
 	};
 
-	LListHead<gprs_rlcmac_tbf> *pos;
+	struct llist_item *pos;
 	gprs_rlcmac_tbf *tbf;
 	unsigned list_idx;
 	struct gprs_rlcmac_tbf **tbf_var;
@@ -70,8 +69,8 @@ static void check_tfi_usage(BTS *the_bts)
 	for (list_idx = 0; list_idx < ARRAY_SIZE(tbf_lists); list_idx += 1)
 	{
 
-		llist_for_each(pos, tbf_lists[list_idx]) {
-			tbf = pos->entry();
+		llist_for_each_entry(pos, tbf_lists[list_idx], list) {
+			tbf = (struct gprs_rlcmac_tbf *)pos->entry;
 			for (pdch_no = 0; pdch_no < 8; pdch_no += 1) {
 				struct gprs_rlcmac_pdch *pdch = tbf->pdch[pdch_no];
 				if (pdch == NULL)
@@ -87,14 +86,14 @@ static void check_tfi_usage(BTS *the_bts)
 				if (tbf->direction == GPRS_RLCMAC_DL_TBF) {
 					OSMO_ASSERT(pdch->dl_tbf_by_tfi(
 							tbf->tfi()) == tbf);
-					OSMO_ASSERT(the_bts->dl_tbf_by_tfi(
+					OSMO_ASSERT(bts_dl_tbf_by_tfi(bts,
 							tbf->tfi(),
 							tbf->trx->trx_no,
 							pdch_no) == tbf);
 				} else {
 					OSMO_ASSERT(pdch->ul_tbf_by_tfi(
 							tbf->tfi()) == tbf);
-					OSMO_ASSERT(the_bts->ul_tbf_by_tfi(
+					OSMO_ASSERT(bts_ul_tbf_by_tfi(bts,
 							tbf->tfi(),
 							tbf->trx->trx_no,
 							pdch_no) == tbf);
@@ -113,14 +112,13 @@ static void test_alloc_a(gprs_rlcmac_tbf_direction dir,
 	int tfi;
 	int i;
 	uint8_t used_trx, tmp_trx;
-	BTS the_bts;
-	struct gprs_rlcmac_bts *bts;
+	struct gprs_rlcmac_bts *bts = bts_alloc(the_pcu, 0);
+	GprsMs *ms;
 	struct gprs_rlcmac_tbf *tbfs[32*8+1] = { 0, };
 
 	printf("Testing alloc_a direction(%d)\n", dir);
 
-	bts = the_bts.bts_data();
-	bts->alloc_algorithm = alloc_algorithm_a;
+	the_pcu->alloc_algorithm = alloc_algorithm_a;
 
 	struct gprs_rlcmac_trx *trx = &bts->trx[0];
 	for (i = 0; i < 8; i += 1)
@@ -136,26 +134,31 @@ static void test_alloc_a(gprs_rlcmac_tbf_direction dir,
 	 * least this part is working okay.
 	 */
 	for (i = 0; i < (int)ARRAY_SIZE(tbfs); ++i) {
-		tbfs[i] = tbf_alloc(bts, NULL, dir, -1, 0, 0, 0);
+		ms = bts_alloc_ms(bts, 0, 0);
+		tbfs[i] = tbf_alloc(bts, ms, dir, -1, 0);
 		if (tbfs[i] == NULL)
 			break;
 
 		used_trx = tbfs[i]->trx->trx_no;
-		tfi = the_bts.tfi_find_free(dir, &tmp_trx, used_trx);
+		tfi = bts_tfi_find_free(bts, dir, &tmp_trx, used_trx);
 		OSMO_ASSERT(tbfs[i]->tfi() != tfi);
 	}
 
-	check_tfi_usage(&the_bts);
+	check_tfi_usage(bts);
 
 	OSMO_ASSERT(i == count);
 
+	OSMO_ASSERT(bts_all_pdch_allocated(bts));
+
 	for (i = 0; i < count; ++i)
 		if (tbfs[i])
 			tbf_free(tbfs[i]);
 
-	tbfs[0] = tbf_alloc(bts, NULL, dir, -1, 0, 0, 0);
+	ms = bts_alloc_ms(bts, 0, 0);
+	tbfs[0] = tbf_alloc(bts, ms, dir, -1, 0);
 	OSMO_ASSERT(tbfs[0]);
 	tbf_free(tbfs[0]);
+	talloc_free(bts);
 }
 
 static void test_alloc_a()
@@ -203,30 +206,32 @@ static inline void enable_ts_on_bts(struct gprs_rlcmac_bts *bts,
 static inline bool test_alloc_b_ul_dl(bool ts0, bool ts1, bool ts2, bool ts3, bool ts4, bool ts5, bool ts6, bool ts7,
 				      uint8_t ms_class, bool verbose)
 {
-	BTS the_bts;
-	struct gprs_rlcmac_bts *bts = the_bts.bts_data();
+	struct gprs_rlcmac_bts *bts = bts_alloc(the_pcu, 0);
+	GprsMs *ms;
 	gprs_rlcmac_ul_tbf *ul_tbf;
 	gprs_rlcmac_dl_tbf *dl_tbf;
 
 	if (verbose)
 		printf("Testing UL then DL assignment.\n");
 
-	bts->alloc_algorithm = alloc_algorithm_b;
+	the_pcu->alloc_algorithm = alloc_algorithm_b;
 
 	enable_ts_on_bts(bts, ts0, ts1, ts2, ts3, ts4, ts5, ts6, ts7);
 
-	ul_tbf = tbf_alloc_ul_tbf(bts, NULL, -1, ms_class, 0, true);
+	ms = bts_alloc_ms(bts, ms_class, 0);
+	/* Avoid delaying free to avoid tons of to-be-freed ms objects queuing */
+	ms_set_timeout(ms, 0);
+	ul_tbf = tbf_alloc_ul_tbf(bts, ms, -1, true);
 	if (!ul_tbf)
 		return false;
 
 	OSMO_ASSERT(ul_tbf->ms());
-	OSMO_ASSERT(ul_tbf->ms()->current_trx());
+	OSMO_ASSERT(ms_current_trx(ul_tbf->ms()));
 
 	dump_assignment(ul_tbf, "UL", verbose);
 
 	/* assume final ack has not been sent */
-	dl_tbf = tbf_alloc_dl_tbf(bts, ul_tbf->ms(), ul_tbf->ms()->current_trx()->trx_no, ms_class, 0,
-				  false);
+	dl_tbf = tbf_alloc_dl_tbf(bts, ms, ms_current_trx(ms)->trx_no, false);
 	if (!dl_tbf)
 		return false;
 
@@ -234,42 +239,43 @@ static inline bool test_alloc_b_ul_dl(bool ts0, bool ts1, bool ts2, bool ts3, bo
 
 	OSMO_ASSERT(dl_tbf->first_common_ts == ul_tbf->first_common_ts);
 
-	check_tfi_usage(&the_bts);
+	check_tfi_usage(bts);
 
 	tbf_free(dl_tbf);
 	tbf_free(ul_tbf);
-
+	talloc_free(bts);
 	return true;
 }
 
 static inline bool test_alloc_b_dl_ul(bool ts0, bool ts1, bool ts2, bool ts3, bool ts4, bool ts5, bool ts6, bool ts7,
 				      uint8_t ms_class, bool verbose)
 {
-	BTS the_bts;
-	struct gprs_rlcmac_bts *bts = the_bts.bts_data();
+	struct gprs_rlcmac_bts *bts = bts_alloc(the_pcu, 0);
+	GprsMs *ms;
 	gprs_rlcmac_ul_tbf *ul_tbf;
 	gprs_rlcmac_dl_tbf *dl_tbf;
 
 	if (verbose)
 		printf("Testing DL then UL assignment followed by update\n");
 
-	bts->alloc_algorithm = alloc_algorithm_b;
+	the_pcu->alloc_algorithm = alloc_algorithm_b;
 
 	enable_ts_on_bts(bts, ts0, ts1, ts2, ts3, ts4, ts5, ts6, ts7);
 
-	dl_tbf = tbf_alloc_dl_tbf(bts, NULL, -1, ms_class, 0, true);
+	ms = bts_alloc_ms(bts, ms_class, 0);
+	/* Avoid delaying free to avoid tons of to-be-freed ms objects queuing */
+	ms_set_timeout(ms, 0);
+	dl_tbf = tbf_alloc_dl_tbf(bts, ms, -1, true);
 	if (!dl_tbf)
 		return false;
 
 	dl_tbf->update_ms(0x23, GPRS_RLCMAC_DL_TBF);
-
-	OSMO_ASSERT(dl_tbf->ms());
-	OSMO_ASSERT(dl_tbf->ms()->current_trx());
+	OSMO_ASSERT(dl_tbf->ms() == ms);
+	OSMO_ASSERT(ms_current_trx(dl_tbf->ms()));
 
 	dump_assignment(dl_tbf, "DL", verbose);
 
-	ul_tbf = tbf_alloc_ul_tbf(bts, dl_tbf->ms(), dl_tbf->ms()->current_trx()->trx_no, ms_class, 0,
-				  false);
+	ul_tbf = tbf_alloc_ul_tbf(bts, ms, ms_current_trx(ms)->trx_no, false);
 	if (!ul_tbf)
 		return false;
 
@@ -285,42 +291,44 @@ static inline bool test_alloc_b_dl_ul(bool ts0, bool ts1, bool ts2, bool ts3, bo
 	dump_assignment(dl_tbf, "DL", verbose);
 	OSMO_ASSERT(dl_tbf->first_common_ts == ul_tbf->first_common_ts);
 
-	check_tfi_usage(&the_bts);
+	check_tfi_usage(bts);
 
 	tbf_free(dl_tbf);
 	tbf_free(ul_tbf);
-
+	talloc_free(bts);
 	return true;
 }
 
 static inline bool test_alloc_b_jolly(uint8_t ms_class)
 {
-	BTS the_bts;
-	struct gprs_rlcmac_bts *bts = the_bts.bts_data();
+	struct gprs_rlcmac_bts *bts = bts_alloc(the_pcu, 0);
+	GprsMs *ms;
 	int tfi;
 	uint8_t trx_no;
 	gprs_rlcmac_tbf *ul_tbf, *dl_tbf;
 
 	printf("Testing jolly example\n");
 
-	bts->alloc_algorithm = alloc_algorithm_b;
+	the_pcu->alloc_algorithm = alloc_algorithm_b;
 
 	enable_ts_on_bts(bts, false, true, true, true, true, false, false, false);
 
-	tfi = the_bts.tfi_find_free(GPRS_RLCMAC_UL_TBF, &trx_no, -1);
+	tfi = bts_tfi_find_free(bts, GPRS_RLCMAC_UL_TBF, &trx_no, -1);
 	OSMO_ASSERT(tfi >= 0);
-	ul_tbf = tbf_alloc_ul_tbf(bts, NULL, .1, ms_class, 0, false);
+	ms = bts_alloc_ms(bts, ms_class, 0);
+	/* Avoid delaying free to avoid tons of to-be-freed ms objects queuing */
+	ms_set_timeout(ms, 0);
+	ul_tbf = tbf_alloc_ul_tbf(bts, ms, -1, false);
 	if (!ul_tbf)
 		return false;
 
-	OSMO_ASSERT(ul_tbf->ms());
-	OSMO_ASSERT(ul_tbf->ms()->current_trx());
-	trx_no = ul_tbf->ms()->current_trx()->trx_no;
+	OSMO_ASSERT(ul_tbf->ms() == ms);
+	OSMO_ASSERT(ms_current_trx(ul_tbf->ms()));
+	trx_no = ms_current_trx(ms)->trx_no;
 	dump_assignment(ul_tbf, "UL", true);
 
 	/* assume final ack has not been sent */
-	dl_tbf = tbf_alloc_dl_tbf(bts, ul_tbf->ms(), trx_no, ms_class, 0,
-				  false);
+	dl_tbf = tbf_alloc_dl_tbf(bts, ms, trx_no, false);
 	if (!dl_tbf)
 		return false;
 
@@ -328,11 +336,11 @@ static inline bool test_alloc_b_jolly(uint8_t ms_class)
 
 	OSMO_ASSERT(dl_tbf->first_common_ts == ul_tbf->first_common_ts);
 
-	check_tfi_usage(&the_bts);
+	check_tfi_usage(bts);
 
 	tbf_free(dl_tbf);
 	tbf_free(ul_tbf);
-
+	talloc_free(bts);
 	return true;
 }
 
@@ -415,9 +423,6 @@ static void test_alloc_b()
 	test_all_alloc_b();
 }
 
-typedef int (*algo_t)(struct gprs_rlcmac_bts *bts, struct GprsMs *ms, struct gprs_rlcmac_tbf *tbf, bool single,
-		      int8_t use_trx);
-
 static char get_dir_char(uint8_t mask, uint8_t tx, uint8_t rx, uint8_t busy)
 {
 	int offs = busy ? 32 : 0;
@@ -449,49 +454,54 @@ static inline char *test_mode_descr(enum test_mode t)
 	}
 }
 
-static GprsMs *alloc_tbfs(BTS *the_bts, GprsMs *ms, unsigned ms_class,
-	enum test_mode mode)
+static GprsMs *alloc_tbfs(struct gprs_rlcmac_bts *bts, struct GprsMs *old_ms, enum test_mode mode)
 {
-	struct gprs_rlcmac_bts *bts;
+	struct GprsMs *ms, *new_ms;
 	uint8_t trx_no = -1;
 
-	bts = the_bts->bts_data();
+	OSMO_ASSERT(old_ms != NULL);
 
 	gprs_rlcmac_tbf *tbf = NULL;
 
-	if (ms && ms->current_trx())
-		trx_no = ms->current_trx()->trx_no;
+	if (ms_current_trx(old_ms))
+		trx_no = ms_current_trx(old_ms)->trx_no;
 
-	GprsMs::Guard guard1(ms);
+	ms_ref(old_ms);
 
 	/* Allocate what is needed first */
 	switch (mode) {
 	case TEST_MODE_UL_ONLY:
 	case TEST_MODE_DL_AFTER_UL:
 	case TEST_MODE_UL_AND_DL:
-		if (ms && ms->ul_tbf())
-			tbf_free(ms->ul_tbf());
-		tbf = tbf_alloc_ul_tbf(bts, ms, trx_no, ms_class, 0, false);
-		if (tbf == NULL)
+		if (ms_ul_tbf(old_ms))
+			tbf_free(ms_ul_tbf(old_ms));
+		tbf = tbf_alloc_ul_tbf(bts, old_ms, trx_no, false);
+		if (tbf == NULL) {
+			OSMO_ASSERT(trx_no != -1 || bts_all_pdch_allocated(bts));
+			ms_unref(old_ms);
 			return NULL;
+		}
 		break;
 	case TEST_MODE_DL_ONLY:
 	case TEST_MODE_UL_AFTER_DL:
 	case TEST_MODE_DL_AND_UL:
-		if (ms && ms->dl_tbf())
-			tbf_free(ms->dl_tbf());
-		tbf = tbf_alloc_dl_tbf(bts, ms, trx_no, ms_class, 0, false);
-		if (tbf == NULL)
+		if (ms_dl_tbf(old_ms))
+			tbf_free(ms_dl_tbf(old_ms));
+		tbf = tbf_alloc_dl_tbf(bts, old_ms, trx_no, false);
+		if (tbf == NULL) {
+			OSMO_ASSERT(trx_no != -1 || bts_all_pdch_allocated(bts));
+			ms_unref(old_ms);
 			return NULL;
+		}
 	}
 
 	OSMO_ASSERT(tbf);
 	OSMO_ASSERT(tbf->ms());
-	OSMO_ASSERT(ms == NULL || ms == tbf->ms());
+	OSMO_ASSERT(old_ms == tbf->ms());
 	ms = tbf->ms();
 
-	GprsMs::Guard guard2(ms);
-
+	ms_ref(ms);
+	new_ms = ms;
 	/* Continue with what is needed next */
 	switch (mode) {
 	case TEST_MODE_UL_ONLY:
@@ -501,12 +511,12 @@ static GprsMs *alloc_tbfs(BTS *the_bts, GprsMs *ms, unsigned ms_class,
 
 	case TEST_MODE_DL_AFTER_UL:
 	case TEST_MODE_UL_AND_DL:
-		ms = alloc_tbfs(the_bts, ms, ms_class, TEST_MODE_DL_ONLY);
+		new_ms = alloc_tbfs(bts, ms, TEST_MODE_DL_ONLY);
 		break;
 
 	case TEST_MODE_UL_AFTER_DL:
 	case TEST_MODE_DL_AND_UL:
-		ms = alloc_tbfs(the_bts, ms, ms_class, TEST_MODE_UL_ONLY);
+		new_ms = alloc_tbfs(bts, ms, TEST_MODE_UL_ONLY);
 		break;
 	}
 
@@ -522,13 +532,15 @@ static GprsMs *alloc_tbfs(BTS *the_bts, GprsMs *ms, unsigned ms_class,
 		break;
 	}
 
-	if (!ms && tbf)
+	if (!new_ms && tbf)
 		tbf_free(tbf);
 
-	return  guard2.is_idle() ? NULL : ms;
+	ms_unref(old_ms);
+	ms_unref(ms);
+	return new_ms;
 }
 
-static unsigned alloc_many_tbfs(BTS *the_bts, unsigned min_class,
+static unsigned alloc_many_tbfs(struct gprs_rlcmac_bts *bts, unsigned min_class,
 	unsigned max_class, enum test_mode mode)
 {
 	unsigned counter;
@@ -548,17 +560,19 @@ static unsigned alloc_many_tbfs(BTS *the_bts, unsigned min_class,
 		enum gprs_rlcmac_tbf_direction dir;
 		uint32_t tlli = counter + 0xc0000000;
 
-		ms = the_bts->ms_by_tlli(tlli);
-
-		ms = alloc_tbfs(the_bts, ms, ms_class, mode);
+		ms = bts_ms_by_tlli(bts, tlli, GSM_RESERVED_TMSI);
+		if (!ms)
+			ms = bts_alloc_ms(bts, 0, 0);
+		ms_set_ms_class(ms, ms_class);
+		ms = alloc_tbfs(bts, ms, mode);
 		if (!ms)
 			break;
 
-		ms->set_tlli(tlli);
+		ms_set_tlli(ms, tlli);
 
-		ul_tbf = ms->ul_tbf();
-		dl_tbf = ms->dl_tbf();
-		trx = ms->current_trx();
+		ul_tbf = ms_ul_tbf(ms);
+		dl_tbf = ms_dl_tbf(ms);
+		trx = ms_current_trx(ms);
 
 		OSMO_ASSERT(ul_tbf || dl_tbf);
 
@@ -609,12 +623,12 @@ static unsigned alloc_many_tbfs(BTS *the_bts, unsigned min_class,
 			get_dir_char(0x80, ul_slots, dl_slots, busy_slots));
 
 		if (tfi >= 0) {
-			OSMO_ASSERT(ms->current_trx());
-			tfi2 = the_bts->tfi_find_free(dir, &trx_no2,
-				ms->current_trx()->trx_no);
+			OSMO_ASSERT(ms_current_trx(ms));
+			tfi2 = bts_tfi_find_free(bts, dir, &trx_no2,
+				ms_current_trx(ms)->trx_no);
 			OSMO_ASSERT(tfi != tfi2);
 			OSMO_ASSERT(tfi2 < 0 ||
-				trx_no2 == ms->current_trx()->trx_no);
+				trx_no2 == ms_current_trx(ms)->trx_no);
 		}
 
 		ms_class += 1;
@@ -625,20 +639,18 @@ static unsigned alloc_many_tbfs(BTS *the_bts, unsigned min_class,
 	return counter;
 }
 
-static void test_successive_allocation(algo_t algo, unsigned min_class,
+static void test_successive_allocation(alloc_algorithm_func_t algo, unsigned min_class,
 	unsigned max_class, enum test_mode mode,
 	unsigned expect_num, const char *text)
 {
-	BTS the_bts;
-	struct gprs_rlcmac_bts *bts;
+	struct gprs_rlcmac_bts *bts = bts_alloc(the_pcu, 0);
 	struct gprs_rlcmac_trx *trx;
 	unsigned counter;
 
 	printf("Going to test assignment with many TBF, algorithm %s class %u..%u (%s)\n",
 	       text, min_class, max_class, test_mode_descr(mode));
 
-	bts = the_bts.bts_data();
-	bts->alloc_algorithm = algo;
+	the_pcu->alloc_algorithm = algo;
 
 	trx = &bts->trx[0];
 	trx->pdch[3].enable();
@@ -647,7 +659,7 @@ static void test_successive_allocation(algo_t algo, unsigned min_class,
 	trx->pdch[6].enable();
 	trx->pdch[7].enable();
 
-	counter = alloc_many_tbfs(&the_bts, min_class, max_class, mode);
+	counter = alloc_many_tbfs(bts, min_class, max_class, mode);
 
 	printf("  Successfully allocated %u UL TBFs, algorithm %s class %u..%u (%s)\n",
 	       counter, text, min_class, max_class, test_mode_descr(mode));
@@ -657,14 +669,14 @@ static void test_successive_allocation(algo_t algo, unsigned min_class,
 
 	OSMO_ASSERT(counter == expect_num);
 
-	check_tfi_usage(&the_bts);
+	check_tfi_usage(bts);
+	talloc_free(bts);
 }
 
-static void test_many_connections(algo_t algo, unsigned expect_num,
+static void test_many_connections(alloc_algorithm_func_t algo, unsigned expect_num,
 	const char *text)
 {
-	BTS the_bts;
-	struct gprs_rlcmac_bts *bts;
+	struct gprs_rlcmac_bts *bts = bts_alloc(the_pcu, 0);
 	struct gprs_rlcmac_trx *trx;
 	int counter1, counter2 = -1;
 	unsigned i;
@@ -677,8 +689,7 @@ static void test_many_connections(algo_t algo, unsigned expect_num,
 
 	printf("Going to test assignment with many connections, algorithm %s\n", text);
 
-	bts = the_bts.bts_data();
-	bts->alloc_algorithm = algo;
+	the_pcu->alloc_algorithm = algo;
 
 	trx = &bts->trx[0];
 	trx->pdch[3].enable();
@@ -688,11 +699,11 @@ static void test_many_connections(algo_t algo, unsigned expect_num,
 	trx->pdch[7].enable();
 
 	for (i = 0; i < ARRAY_SIZE(mode_seq); i += 1) {
-		counter1 = alloc_many_tbfs(&the_bts, 1, mslot_class_max(), mode_seq[i]);
+		counter1 = alloc_many_tbfs(bts, 1, mslot_class_max(), mode_seq[i]);
 		fprintf(stderr, "  Allocated %d TBFs (previously %d)\n",
 			counter1, counter2);
 
-		check_tfi_usage(&the_bts);
+		check_tfi_usage(bts);
 
 		/* This will stop earlier due to USF shortage */
 		if (mode_seq[i] == TEST_MODE_UL_ONLY)
@@ -713,6 +724,7 @@ static void test_many_connections(algo_t algo, unsigned expect_num,
 		fprintf(stderr, "  Expected %d TBFs (got %d) for algorithm %s\n", expect_num, counter1, text);
 
 	OSMO_ASSERT(expect_num == (unsigned)counter1);
+	talloc_free(bts);
 }
 
 static inline void test_a_b_dyn(enum test_mode mode, uint8_t exp_A, uint8_t exp_B, uint8_t exp_dyn)
@@ -725,24 +737,24 @@ static inline void test_a_b_dyn(enum test_mode mode, uint8_t exp_A, uint8_t exp_
 static void test_successive_allocations()
 {
 	test_successive_allocation(alloc_algorithm_a,       1,  1, TEST_MODE_UL_AND_DL, 35, "A");
-	test_successive_allocation(alloc_algorithm_b,      10, 10, TEST_MODE_UL_AND_DL, 32, "B");
-	test_successive_allocation(alloc_algorithm_b,      12, 12, TEST_MODE_UL_AND_DL, 32, "B");
+	test_successive_allocation(alloc_algorithm_b,      10, 10, TEST_MODE_UL_AND_DL, 15, "B");
+	test_successive_allocation(alloc_algorithm_b,      12, 12, TEST_MODE_UL_AND_DL, 15, "B");
 
-	test_successive_allocation(alloc_algorithm_b,       1,                12, TEST_MODE_UL_AND_DL, 32, "B");
-	test_successive_allocation(alloc_algorithm_b,       1, mslot_class_max(), TEST_MODE_UL_AND_DL, 32, "B");
-	test_successive_allocation(alloc_algorithm_dynamic, 1, mslot_class_max(), TEST_MODE_UL_AND_DL, 35, "dynamic");
+	test_successive_allocation(alloc_algorithm_b,       1,                12, TEST_MODE_UL_AND_DL, 23, "B");
+	test_successive_allocation(alloc_algorithm_b,       1, mslot_class_max(), TEST_MODE_UL_AND_DL, 17, "B");
+	test_successive_allocation(alloc_algorithm_dynamic, 1, mslot_class_max(), TEST_MODE_UL_AND_DL, 17, "dynamic");
 
-	test_a_b_dyn(TEST_MODE_DL_AND_UL,    35, 32,  32);
-	test_a_b_dyn(TEST_MODE_DL_AFTER_UL, 160, 32,  95);
-	test_a_b_dyn(TEST_MODE_UL_AFTER_DL,  35, 32,  35);
-	test_a_b_dyn(TEST_MODE_UL_ONLY,      35, 32,  35);
+	test_a_b_dyn(TEST_MODE_DL_AND_UL,    35, 15,  15);
+	test_a_b_dyn(TEST_MODE_DL_AFTER_UL, 160, 32,  101);
+	test_a_b_dyn(TEST_MODE_UL_AFTER_DL,  35, 15,  15);
+	test_a_b_dyn(TEST_MODE_UL_ONLY,      35, 15,  21);
 	test_a_b_dyn(TEST_MODE_DL_ONLY,     160, 32, 101);
 }
 
 static void test_2_consecutive_dl_tbfs()
 {
-	BTS the_bts;
-	struct gprs_rlcmac_bts *bts;
+	struct gprs_rlcmac_bts *bts = bts_alloc(the_pcu, 0);
+	GprsMs *ms;
 	struct gprs_rlcmac_trx *trx;
 	uint8_t ms_class = 11;
 	uint8_t egprs_ms_class = 11;
@@ -751,8 +763,7 @@ static void test_2_consecutive_dl_tbfs()
 
 	printf("Testing DL TS allocation for Multi UEs\n");
 
-	bts = the_bts.bts_data();
-	bts->alloc_algorithm = alloc_algorithm_b;
+	the_pcu->alloc_algorithm = alloc_algorithm_b;
 
 	trx = &bts->trx[0];
 	trx->pdch[4].enable();
@@ -760,8 +771,8 @@ static void test_2_consecutive_dl_tbfs()
 	trx->pdch[6].enable();
 	trx->pdch[7].enable();
 
-	dl_tbf1 = tbf_alloc_dl_tbf(bts, NULL, 0, ms_class, egprs_ms_class,
-				   false);
+	ms = bts_alloc_ms(bts, ms_class, egprs_ms_class);
+	dl_tbf1 = tbf_alloc_dl_tbf(bts, ms, 0, false);
 	OSMO_ASSERT(dl_tbf1);
 
 	for (int i = 0; i < 8; i++) {
@@ -771,8 +782,8 @@ static void test_2_consecutive_dl_tbfs()
 	OSMO_ASSERT(numTs1 == 4);
 	printf("TBF1: numTs(%d)\n", numTs1);
 
-	dl_tbf2 = tbf_alloc_dl_tbf(bts, NULL, 0, ms_class, egprs_ms_class,
-				   false);
+	ms = bts_alloc_ms(bts, ms_class, egprs_ms_class);
+	dl_tbf2 = tbf_alloc_dl_tbf(bts, ms, 0, false);
 	OSMO_ASSERT(dl_tbf2);
 
 	for (int i = 0; i < 8; i++) {
@@ -789,6 +800,31 @@ static void test_2_consecutive_dl_tbfs()
 
 	tbf_free(dl_tbf1);
 	tbf_free(dl_tbf2);
+	talloc_free(bts);
+}
+
+static void test_bts_pch_timer(void)
+{
+	struct gprs_rlcmac_bts *bts = bts_alloc(the_pcu, 0);
+	struct osmo_mobile_identity mi_imsi1, mi_imsi2;
+	struct osmo_mobile_identity  mi_tmsi1;
+	mi_imsi1.type = mi_imsi2.type = GSM_MI_TYPE_IMSI;
+	mi_tmsi1.type = GSM_MI_TYPE_TMSI;
+	OSMO_STRLCPY_ARRAY(mi_imsi1.imsi, "1234");
+	OSMO_STRLCPY_ARRAY(mi_imsi2.imsi, "5678");
+	mi_tmsi1.tmsi = 987654321;
+
+	fprintf(stderr, "Testing bts_pch_timer dealloc on bts dealloc\n");
+	log_set_category_filter(osmo_stderr_target, DPCU, 1, LOGL_DEBUG);
+
+	fprintf(stderr, "Starting PCH timer for 2 IMSI\n");
+	bts_pch_timer_start(bts, &mi_imsi1, mi_imsi1.imsi);
+	bts_pch_timer_start(bts, &mi_imsi2, mi_imsi2.imsi);
+	fprintf(stderr, "Starting PCH timer for 1 TMSI\n");
+	bts_pch_timer_start(bts, &mi_tmsi1, "6666");
+
+	fprintf(stderr, "Deallocating BTS, expecting the PCH timer to be stopped and deallocated\n");
+	talloc_free(bts);
 }
 
 int main(int argc, char **argv)
@@ -800,9 +836,15 @@ int main(int argc, char **argv)
 	msgb_talloc_ctx_init(tall_pcu_ctx, 0);
 	osmo_init_logging2(tall_pcu_ctx, &gprs_log_info);
 	log_set_use_color(osmo_stderr_target, 0);
-	log_set_print_filename(osmo_stderr_target, 0);
+	log_set_print_filename2(osmo_stderr_target, LOG_FILENAME_NONE);
+	log_set_print_category(osmo_stderr_target, 0);
+	log_set_print_category_hex(osmo_stderr_target, 0);
+	log_set_category_filter(osmo_stderr_target, DTBF, 1, LOGL_INFO);
 	if (getenv("LOGL_DEBUG"))
 		log_set_log_level(osmo_stderr_target, LOGL_DEBUG);
+	osmo_fsm_log_addr(false);
+
+	the_pcu = gprs_pcu_alloc(tall_pcu_ctx);
 
 	test_alloc_a();
 	test_alloc_b();
@@ -811,6 +853,9 @@ int main(int argc, char **argv)
 	test_many_connections(alloc_algorithm_b, 32, "B");
 	test_many_connections(alloc_algorithm_dynamic, 160, "dynamic");
 	test_2_consecutive_dl_tbfs();
+	test_bts_pch_timer();
+
+	talloc_free(the_pcu);
 	return EXIT_SUCCESS;
 }
 
